
//------------------------------------------------------------------------------
//                                                                            --
//     USE OF SOFTWARE. This software contains elements of software code      --
//     which is the property of Nallatech Limited (the "Nallatech Software"). --
//     Use of the Nallatech Software by you is permitted only if you hold a   --
//     valid license from Nallatech Limited or a valid sub-license from a     --
//     licensee of Nallatech Limited. Use of such software shall be governed  --
//     by the terms of such license or sub-license agreement.                 --
//                                                                            --
//     This Nallatech Software is protected by copyright law, patent law and  --
//     international treaties. Unauthorized reproduction or distribution of   --
//     this software, or any portion of it, may result in severe civil and    --
//     criminal penalties, and will be prosecuted to the maximum extent       --
//     possible under law.                                                    --
//                                                                            --
//     (c) Copyright 2001 - 2005 Nallatech Limited.                           --
//     All rights reserved.                                                   --
//                                                                            --
//------------------------------------------------------------------------------
//
// Requires import libraries and headers.
// dimesdl.lib imports dimesdl functions from dimesdl.dll
// dimetalk.lib imports dimetalk functions from dimetalk.dll
// dimesdl.h contains prototypes for FUSE API functions.
// dimetalk.h contains prototypes for DIMETalk API functions.
// All of these files can both be found in the FUSE\include directory

#include <stdio.h>
#include "nalla_wrapper.h"
#include "tsc.h"
#include <time.h>
#include <stdlib.h>
#include <assert.h>

#define MEM_ID 1
#define NUM_TESTS 1000
#define ADDR_WIDTH 15
#define MAX_SIZE (1<<ADDR_WIDTH)
#define MEM_IN_ADDR 0x0010
#define MEM_OUT_ADDR (MEM_IN_ADDR << ADDR_WIDTH)

typedef enum MAP {
  GO_ADDR=0, SIZE_ADDR, DONE_ADDR
} MAP;


void sw(unsigned int *input, unsigned int *output, unsigned int size) {
  
  unsigned int i;

  for (i=0; i < size; i++) {

      unsigned int in1, in2, in3, in4;
      in1 = (input[i] >> 24) & 0xff;
      in2 = (input[i] >> 16) & 0xff;
      in3 = (input[i] >> 8) & 0xff;
      in4 = (input[i]) & 0xff;

      output[i] = in1*in2 + in3*in4;
  }
}

double timerval();
int main(int argc, char* argv[])
{
    char *myfile = "h101_pcixm_0.bit";
    double clkA = 100.0;
    double clkB = 80.0;
    double clkC = 150.0;

    FPGA_init(clkA, clkB, clkC);
    FPGA_config(myfile);

    unsigned int go, done;
    unsigned int i,j;
    unsigned int size;
    unsigned int *input;
    unsigned int *swOutput;
    unsigned int *hwOutput;
    unsigned long long diff = 0;

    size = MAX_SIZE;
    FPGA_write(&size, 1, SIZE_ADDR, MEM_ID, 1000);

    input = (unsigned int*) malloc(size*sizeof(unsigned int));
    hwOutput = (unsigned int*) malloc(size*sizeof(unsigned int));
    swOutput = (unsigned int*) malloc(size*sizeof(unsigned int));
    assert(input != NULL);
    assert(swOutput != NULL);
    assert(hwOutput != NULL);

    for (i=0; i < size; i++) {

        input[i] = ((i*4) & 0xff) << 24 |
            ((i*4+1) & 0xff) << 16 |
            ((i*4+2) & 0xff) << 8 |
            ((i*4+3) & 0xff);
        swOutput[i] = 0;
        hwOutput[i] = 0;
    }

    sw(input,swOutput,size);
    FPGA_write(input, size, MEM_IN_ADDR, MEM_ID, 1000);

    for (i=0; i < NUM_TESTS; i++) {

        printf("Test %d...\n", i);
        fflush(stdout);

        go = 1;
        FPGA_write(&go, 1, GO_ADDR, MEM_ID, 1000);
        
        done = 0;
        while (!done) {
            FPGA_read(&done, 1, DONE_ADDR, MEM_ID, 1000);
        }
        
        FPGA_read(hwOutput, size, MEM_OUT_ADDR, MEM_ID, 1000);
        
        for (j=0; j < size; j++) {
//            printf("output[%d] = %d\n", j, output[j]);
            diff += abs(hwOutput[j]-swOutput[j]);
        }
    }

    printf("Avg difference: %f\n", (float) (diff/(float) NUM_TESTS));  
    free(input);
    free(hwOutput);
    free(swOutput);
    FPGA_release();
    return 0;
}

double timerval()
{
    struct timeval st;
    gettimeofday(&st,NULL);
    return st.tv_sec + st.tv_usec*1e-6;
}
